from openravepy import *
from time import sleep
import gcode2contour as gc
from math import pi
import numpy as np
from numpy import array
from numpy import matrix as mat
#from numpy.linalg import inv

def world2printTransform(point, Tr):
    '''
    world2printTransform
    - This will convert world coordinates to robot coordinates.
    - Additionally, it will set the angle of approach to be pointing down at the table

    NOTE: This works when robot transforms are lateral, but not for rotations
    
    Inputs:
    - point: [0.3, 0, 1.1]
    - Transformation matrix of robot relative to environment: (numpy matrix)

    Output:
    - Transformation matrix (numpy matrix) which can be fed directly to IK solver
    '''
    point.append(1)
    return np.concatenate(
            (mat([[1,0,0],[0,-1,0],[0,0,-1],[0,0,0]]), Tr * mat(point).T),
            axis=1
            )

def interp_pose(pose1, pose2, t1, t2, ti):
    """
    Linear interpolation of a pose between two times
    
    Inputs:
    - pose1 (list)
    - pose2 (list)
    - t1 (time at pose1)
    - t2 (time at pose2)
    - ti (time to interpolate for)

    Output:
    - pose_i (list)
    """
    temp = (ti-t1)/(t2-t1)
    return [p1 + (p2-p1)*temp for p1,p2 in zip(pose1, pose2)]


def show_contour(contour, env):
    """
    Display contour line in visualiser
    Input: contour object (gcode2contour)
    """
    return env.drawlinestrip(
            points = array(contour.pos),
            linewidth = 3,
            colors=array((0,0,0))
            )
